/* g++ -DLINUX examples/auxversion_example.cpp  -lcc-aux2 -pthread -ldl */
#include <assert.h>
#include <AuxVersion.h>
#include <iomanip>
#include <iostream>

using namespace std;
using namespace CrossControl;

void output_versions(AUXVERSIONHANDLE pAuxVersion)
{
  if (!pAuxVersion)
    return;

  int const column_width = 32;
  unsigned char major, minor, release, build;
  CrossControl::eErr err;

  //! [Calling AuxVersion_getFPGAVersion]
  err = AuxVersion_getFPGAVersion(
    pAuxVersion,
    &major,
    &minor,
    &release,
    &build);

  cout << setw(column_width) << "FPGA Version: ";
  if (CrossControl::ERR_SUCCESS == err)
    cout << (int) major << "." <<
    (int) minor << "." <<
    (int) release << "." <<
    (int) build << endl;
  else
    cout << "unknown" << endl;
  //! [Calling AuxVersion_getFPGAVersion]

  //! [Calling AuxVersion_getSSVersion]
  err = AuxVersion_getSSVersion(
    pAuxVersion,
    &major,
    &minor,
    &release,
    &build);

  cout << setw(column_width) << "System Supervisor Version: ";
  if (CrossControl::ERR_SUCCESS == err)
    cout << (int) major << "." <<
    (int) minor << "." <<
    (int) release << "." <<
    (int) build << endl;
  else
    cout << "unknown" << endl;
  //! [Calling AuxVersion_getSSVersion]

  //! [Calling AuxVersion_getFrontVersion]
  err = AuxVersion_getFrontVersion(
    pAuxVersion,
    &major,
    &minor,
    &release,
    &build);

  cout << setw(column_width) << "Front Micro Controller Version: ";
  if (CrossControl::ERR_SUCCESS == err)
    cout << (int) major << "." <<
    (int) minor << "." <<
    (int) release << "." <<
    (int) build << endl;
  else
    cout << "unknown" << endl;
  //! [Calling AuxVersion_getFrontVersion]

  //! [Calling AuxVersion_getCCAuxVersion]
  err = AuxVersion_getCCAuxVersion(
    pAuxVersion,
    &major,
    &minor,
    &release,
    &build);

  cout << setw(column_width) << "CC Aux Version: ";
  if (CrossControl::ERR_SUCCESS == err)
    cout <<
    (int) major << "." <<
    (int) minor << "." <<
    (int) release << "." <<
    (int) build << endl;
  else
    cout << "unknown" << endl;
  //! [Calling AuxVersion_getCCAuxVersion]

  //! [Calling AuxVersion_getOSVersion]
  err = AuxVersion_getOSVersion(
    pAuxVersion,
    &major,
    &minor,
    &release,
    &build);

  cout << setw(column_width) << "Operating System Version: ";
  if (CrossControl::ERR_SUCCESS == err)
    cout << (int) major << "." <<
    (int) minor << "." <<
    (int) release << "." <<
    (int) build << endl;
  else
    cout << "unknown" << endl;
  //! [Calling AuxVersion_getOSVersion]

  //! [Calling AuxVersion_getCCAuxDrvVersion]
  err = AuxVersion_getCCAuxDrvVersion(
    pAuxVersion,
    &major,
    &minor,
    &release,
    &build);

  cout << setw(column_width) << "CCAux Driver Version: ";
  if (CrossControl::ERR_SUCCESS == err)
    cout << (int) major << "." <<
    (int) minor << "." <<
    (int) release << "." <<
    (int) build << endl;
  else
    cout << "unknown" << endl;
  //! [Calling AuxVersion_getCCAuxDrvVersion]
}

int main(void)
{
  //! [Calling AuxVersion_release]
  //! [Calling GetAuxVersion]
  AUXVERSIONHANDLE pAuxVersion = ::GetAuxVersion();
  assert (pAuxVersion);

  output_versions(pAuxVersion);

  AuxVersion_release(pAuxVersion);
  //! [Calling GetAuxVersion]
  //! [Calling AuxVersion_release]

  return 0;
}
