/* g++ -DLINUX examples/backlight_example.cpp  -lcc-aux2 -pthread -ldl */
#include <Backlight.h>
#include <CCAuxErrors.h>
#include <assert.h>
#include <iostream>
#include <stdio.h>
#include <stdlib.h>

using namespace std;
using namespace CrossControl;

void change_backlight(BACKLIGHTHANDLE pBacklight, int val)
{
  if (!pBacklight)
    return;

  CrossControl::eErr err;
  unsigned char value;

  //! [Calling Backlight_getStatus]
  err = Backlight_getStatus(pBacklight, &value);
  if (err == ERR_SUCCESS)
  {
    printf("Backlight status: \nBL1:%s\nBL2:%s\nBL3:%s\nBL4:%s\n",
      (value & 0x01)? "OK" : "NOT OK or missing",  
      (value & 0x02)? "OK" : "NOT OK or missing",  
      (value & 0x04)? "OK" : "NOT OK or missing",  
      (value & 0x08)? "OK" : "NOT OK or missing");
  }
  else if (err == ERR_NOT_SUPPORTED)
  {
    printf("Backlight_getStatus: Not supported!\n");  
  }
  else
  {
    printf("Error(%d) in function Backlight_getStatus: %s\n", err, GetErrorStringA(err));
  }
  //! [Calling Backlight_getStatus]

  //! [Calling Backlight_getHWStatus]
  bool backlightStatus = false;
  err = Backlight_getHWStatus(pBacklight, &backlightStatus);
  if (err == ERR_SUCCESS)
  {
    if (backlightStatus)
      printf("Backlight hardware status: OK\n"); 
    else
      printf("Backlight hardware status: not OK, one or more backlight drivers are faulty\n"); 
  }
  else if (err == ERR_NOT_SUPPORTED)
  {
    printf("Backlight_getHWStatus: Not supported!\n");  
  }
  else
  {
    printf("Error(%d) in function Backlight_getHWStatus: %s\n", err, GetErrorStringA(err));
  }
  //! [Calling Backlight_getHWStatus]

  //! [Calling Backlight_getIntensity]
  err = Backlight_getIntensity(pBacklight, &value);

  if (err == ERR_SUCCESS)
  {
    printf("Current backlight intensity (0-255): %d\n", value);
  }
  else
  {
    printf("Error(%d) in function Backlight_getIntensity: %s\n", err, GetErrorStringA(err));
  }
  //! [Calling Backlight_getIntensity]

  if(val < 0)
  {
    if (value < 245)
      value = value + 10;
    else
      value = value -10;
  }
  else
  {
    value = (unsigned char) val;
  }
  //! [Calling Backlight_setIntensity]
  err = Backlight_setIntensity(pBacklight, value);

  if (err == ERR_SUCCESS)
  {
    printf("Setting backlight intensity: %d\n", value);
  }
  else
  {
    printf("Error(%d) in function Backlight_setIntensity: %s\n", err, GetErrorStringA(err));
  }
  //! [Calling Backlight_setIntensity]

}

int main(int argc, char* argv[])
{
  //! [Calling GetBacklight] 
  //! [Calling Backlight_release]
  BACKLIGHTHANDLE pBacklight = ::GetBacklight();
  assert(pBacklight);

  if(argc == 2) 
  {
    change_backlight(pBacklight, (unsigned char)atoi(argv[1]));
  }
  else
  {
    change_backlight(pBacklight, -1);
  }

  Backlight_release(pBacklight);
  //! [Calling Backlight_release]
  //! [Calling GetBacklight]
  
  return 0;
}
