/* g++ -DLINUX examples/buttons_example.cpp  -lcc-aux2 -pthread -ldl */
#ifdef LINUX

#include <Config.h>
#include <CCAuxErrors.h>
#include <assert.h>
#include <iostream>
#include <stdlib.h>
#include <stdio.h>
#include <sys/signal.h>
#include <sys/types.h>
#include <sys/time.h>
#include <linux/input.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <ctype.h>

using namespace std;
using namespace CrossControl;

void buttons_example(CONFIGHANDLE pConfig)
{
  struct input_event event;
  fd_set set;
  struct timeval timeout;
  int fd = 0;
  int err;

  if (!pConfig)
    return;

  //! [Calling Config_getButtonFunction]
  CrossControl::ButtonConfigEnum btnconf;
  CrossControl::eErr error;
  for (int i = 1; i < 9; i++)
  {
    error = Config_getButtonFunction(pConfig, i, &btnconf);
    if (error != ERR_SUCCESS)
    {
      cout << "Error(" << error << ") in function Config_getButtonFunction: " << GetErrorStringA(error) << std::endl;
    }
    else
    {
      cout << "Button " << (int)i << " is set to: ";
      switch(btnconf)
      {
      case BUTTON_ONLY_MP_ACTION: cout << "Application only" << std::endl; break;
      case BUTTON_AS_STARTUP_TRIG:  cout << "Startup trigger" << std::endl; break;
      case BUTTON_AS_ACTION_TRIG: cout << "Action trigger" << std::endl; break;
      case BUTTON_AS_ACTION_STARTUP_TRIG: cout << "Action and Startup trigger" << std::endl; break;
      case BUTTON_AS_BACKLIGHT_DECREASE: cout << "Backlight decrease" << std::endl; break;
      case BUTTON_AS_BACKLIGHT_DECR_STARTUP_TRIG: cout << "Backlight decrease and Startup trigger" << std::endl; break;
      case BUTTON_AS_BACKLIGHT_INCREASE: cout << "Backlight increase" << std::endl; break;
      case BUTTON_AS_BACKLIGHT_INCR_STARTUP_TRIG: cout << "Backlight increase and Startup trigger" << std::endl; break;
      default: cout << "Invalid value" << std::endl; break;
      }
    }
  }
  //! [Calling Config_getButtonFunction]

  cout << std::endl;
  cout << "This example shows how to read CCpilot VC buttons from a c/c++ application." << std::endl;
  cout << "The code also contains QT code example." << std::endl;
  cout << "Press Ctrl-C to exit" << std::endl;

  fd = open("/dev/input/event0", O_RDONLY);

  if (!fd)
  {
    cout << "Open /dev/input/event0 failed!!" << std::endl;
    return;
  }

  // Initialize the timeout data structure
  timeout.tv_sec = 0;
  timeout.tv_usec = 10*1000; // 10 ms

  while (true)
  {
    // Initialize file descriptor set
    FD_ZERO(&set);
    FD_SET(fd, &set);

    // select returns 0 on timeout, 1 if input available, -1 if error
    err = select(FD_SETSIZE, &set, NULL, NULL, &timeout);

    if (err == 1) // Input available	
    {
      // Read event
      if (sizeof(event) != read(fd, &event, sizeof(event)))
      {
        cout << "Read event failed in read button thread!" << std::endl;
        close(fd);
        return;
      }

      // Check if the event matches a key
      if (event.type == EV_KEY)
      {
        switch(event.code)
        {
        case KEY_F1:
          if (event.value == 1)
            cout << "Button 1 pressed!" << std::endl;
          else
            cout << "Button 1 released!" << std::endl;
          break;
        case KEY_F2:
          if (event.value == 1)
            cout << "Button 2 pressed!" << std::endl;
          else
            cout << "Button 2 released!" << std::endl;
          break;					
        case KEY_F3:
          if (event.value == 1)
            cout << "Button 3 pressed!" << std::endl;
          else
            cout << "Button 3 released!" << std::endl;
          break;
        case KEY_F4:
          if (event.value == 1)
            cout << "Button 4 pressed!" << std::endl;
          else
            cout << "Button 4 released!" << std::endl;
          break;			
        case KEY_F5:
          if (event.value == 1)
            cout << "Button 5 pressed!" << std::endl;
          else
            cout << "Button 5 released!" << std::endl;
          break;
        case KEY_F6:
          if (event.value == 1)
            cout << "Button 6 pressed!" << std::endl;
          else
            cout << "Button 6 released!" << std::endl;
          break;
        case KEY_F7:
          if (event.value == 1)
            cout << "Button 7 pressed!" << std::endl;
          else
            cout << "Button 7 released!" << std::endl;
          break;
        case KEY_F8:
          if (event.value == 1)
            cout << "Button 8 pressed!" << std::endl;
          else
            cout << "Button 8 released!" << std::endl;
          break;
        }  
      }
    }
    else if (err == 0)
    {
      // Timeout
    }
    else
    {
      // Error
      cout << "select failed!" << std::endl;
      close(fd);
      return;
    }
  } // while (true)
}

/*
 
 Corresponding example code for QT applications:
 
#include <linux/input.h>
#include <fcntl.h>
#include <unistd.h>
#include <QSocketNotifier>

 QSocketNotifier notifier;
 int buttonsFd;

 example::example()
 {
  const QString &device("/dev/input/event0");
  buttonsFd = ::open(device.toLocal8Bit().constData(), O_RDONLY, 0);
  if (!buttonsFd)
  {
    // "Failed to open input device"
  } 
  else 
  {
    notifier = new QSocketNotifier(buttonsFd, QSocketNotifier::Read, this);
    connect(notifier, SIGNAL(activated(int)), this, SLOT(processButtonEvent()));
  }
 }

void processButtonEvent()
{
  struct input_event event;
  int n = read(buttonsFd, &event, sizeof(input_event));
  if (n != 16) 
  {
    // "Unexpected lenght read"
    return;
  }

  if (event.type != EV_KEY)
    return;

  switch (event.code) 
  {
    case KEY_F1:
       if (event.value == 1)
            cout << "Button 1 pressed!" << std::endl;
          else
            cout << "Button 1 released!" << std::endl;
      break;

      ...
  }
}

 */

int main(void)
{
  CONFIGHANDLE pConfig = ::GetConfig();
  assert(pConfig);

  buttons_example(pConfig);

  Config_release(pConfig);
  
  return 0;
}

#endif // #ifdef LINUX
