/* g++ -DLINUX examples/cansetting_example.cpp  -lcc-aux2 -pthread -ldl */
#include <CanSetting.h>
#include <About.h>
#include <CCAuxErrors.h>
#include <assert.h>
#include <iostream>

using namespace std;
using namespace CrossControl;

void read_cansettings(CANSETTINGHANDLE pCanSetting)
{
  CrossControl::eErr err;
  unsigned short baudrates[4];
#ifndef LINUX
  CrossControl::CanFrameType frametypes[4];
#endif
  unsigned char net;

  if (!pCanSetting)
    return;

  unsigned char nrCan = 0; 
  //Find out number of Can
  ABOUTHANDLE pAbout = ::GetAbout();
  err = About_getNrOfCANConnections(pAbout, &nrCan);
  About_release(pAbout);

  if (err != ERR_SUCCESS)
    {
      nrCan = 0;
      cout << "Error(" << err << ") in function About_getNrOfCANConnections: " << GetErrorStringA(err) << endl;
    }
  
  for(net = 1; net <= nrCan; net++)
  {
    //! [Calling CanSetting_getBaudrate]
    err = CanSetting_getBaudrate(pCanSetting, net, &baudrates[net-1]);
    if (err != ERR_SUCCESS) 
    {
      cout << "Error(" << err << ") in function getBaudrate: " << GetErrorStringA(err) << endl;
      break;
    } 
    //! [Calling CanSetting_getBaudrate]

#ifndef LINUX
    //! [Calling CanSetting_getFrameType]
    err = CanSetting_getFrameType(pCanSetting, net, &frametypes[net-1]);
    if (err != ERR_SUCCESS) 
    {
      cout << "Error(" << err << ") in function getFrameType: " << GetErrorStringA(err) << endl;
      break;
    }  
    //! [Calling CanSetting_getFrameType]
#endif
  }

  if (err == ERR_SUCCESS) 
  {
    for(net = 1; net <= nrCan; net++)
    {
      cout << "Can" << (int)net << ": " << (int)baudrates[net-1] << "kbit/s";
#ifndef LINUX
      switch(frametypes[net-1])
      {
      case FrameStandard:  cout << ", Standard" << endl; break;
      case FrameExtended:  cout << ", Extended" << endl; break;
      case FrameStandardExtended: cout << ", Standard/Extended" << endl; break;
      default: cout << ", Undefined Frametype" << endl; break;
      }
#endif
      cout << std::endl;
    }
  }  
}

int main(void)
{
  //! [Calling CanSetting_release]
  //! [Calling GetCanSetting]
  CANSETTINGHANDLE pCanSetting = ::GetCanSetting();
  assert(pCanSetting);

  read_cansettings(pCanSetting);

  CanSetting_release(pCanSetting);
  //! [Calling GetCanSetting]
  //! [Calling CanSetting_release]

  return 0;
}
