/* g++ -DLINUX examples/cfgin_example.cpp  -lcc-aux2 -pthread -ldl */
#include <CfgIn.h>
#include <CCAuxErrors.h>
#include <assert.h>
#include <iostream>

using namespace std;
using namespace CrossControl;

void cfgin_example(CFGINHANDLE pCfgIn)
{
  CrossControl::eErr err;
  unsigned char channel = 1;
  CfgInModeEnum get_mode;

  if (!pCfgIn)
    return;

  //
  // Set mode on CI 1 to CFGIN_VOLTAGE_2V5 and read value
  // Set minimum frequency threshold to 50 Hz for CI2
  // Set mode on CI 2 to CFGIN_FREQ_PULLDOWN and read values
  //
  channel = 1;

  //! [Calling CfgIn_setCfgInMode]
  err = CfgIn_setCfgInMode(pCfgIn, channel, CFGIN_VOLTAGE_2V5);
  if (err != ERR_SUCCESS)
  {
    cout << "CfgIn_setCfgInMode: " << GetErrorStringA(err) << std::endl;
  }
  else
  {
    cout << "CfgIn_setCfgInMode: channel 1 mode set to CFGIN_VOLTAGE_2V5" << std::endl;
  }
  //! [Calling CfgIn_setCfgInMode]

  channel = 2;
  err = CfgIn_setCfgInMode(pCfgIn, channel, CFGIN_FREQ_PULLDOWN);
  if (err != ERR_SUCCESS)
  {
    cout << "CfgIn_setCfgInMode: " << GetErrorStringA(err) << std::endl;
  }
  else
  {
    cout << "CfgIn_setCfgInMode: channel 2 mode set to CFGIN_FREQ_PULLDOWN" << std::endl;
  }

  //! [Calling CfgIn_getMinFrequencyThreshold]
  float frequency_threshold;
  channel = 2;
  err = CfgIn_getMinFrequencyThreshold(pCfgIn, channel, &frequency_threshold);
  if (err != ERR_SUCCESS)
  {
    cout << "CfgIn_getMinFrequencyThreshold: " << GetErrorStringA(err) << std::endl;
  }
  else
  {
    cout << "CfgIn_getMinFrequencyThreshold: channel 2: " << std::fixed << frequency_threshold << "Hz" << std::endl;
  }
  //! [Calling CfgIn_getMinFrequencyThreshold]

  //! [Calling CfgIn_setMinFrequencyThreshold]
  channel = 2;
  err = CfgIn_setMinFrequencyThreshold(pCfgIn, channel, 50.0);
  if (err != ERR_SUCCESS)
  {
    cout << "CfgIn_setMinFrequencyThreshold: " << GetErrorStringA(err) << std::endl;
  }
  else
  {
    cout << "CfgIn_setMinFrequencyThreshold: channel 2 minimum frequency threshold set to 50.0Hz" << std::endl;
  }
  //! [Calling CfgIn_setMinFrequencyThreshold]

  //! [Calling CfgIn_getValue]
  unsigned short value;
  err = CfgIn_getValue(pCfgIn, 1, &value);
  if (err != ERR_SUCCESS)
  {
    cout << "CfgIn_getValue: " << GetErrorStringA(err) << std::endl;
  }
  else
  {
    cout << "CfgIn_getValue: channel 1 2V5 voltage measurement: " << (int)value << "mV" << std::endl;
  }
  //! [Calling CfgIn_getValue]

  //! [Calling CfgIn_getPwmValue]
  float frequency;
  unsigned char duty_cycle;
  err = CfgIn_getPwmValue(pCfgIn, 2, &frequency, &duty_cycle);
  if (err != ERR_SUCCESS)
  {
    cout << "CfgIn_getPwmValue: " << GetErrorStringA(err) << std::endl;
  }
  else
  {
    cout << "CfgIn_getPwmValue: channel 2 PWM measurement: " << std::fixed << frequency << "Hz, " << (int)duty_cycle << "% duty cycle" << std::endl;
  }
  //! [Calling CfgIn_getPwmValue]

  //
  // Read the modes
  //
  channel = 1;
  while(channel <= 2)
  {
    //! [Calling CfgIn_getCfgInMode]
    err = CfgIn_getCfgInMode(pCfgIn, channel, &get_mode);
    if (err != ERR_SUCCESS)
    {
      cout << "CfgIn_getCfgInMode: " << GetErrorStringA(err) << std::endl;
    }
    else
    {
      switch(get_mode)
      {        
      case CFGIN_NOT_IN_USE: cout << "CfgIn_getCfgInMode (" << (int)channel << "): CFGIN_NOT_IN_USE" << std::endl; break;
      case CFGIN_HI_SWITCH: cout << "CfgIn_getCfgInMode (" << (int)channel << "): CFGIN_HI_SWITCH" << std::endl; break;
      case CFGIN_LOW_SWITCH: cout << "CfgIn_getCfgInMode (" << (int)channel << "): CFGIN_LOW_SWITCH" << std::endl; break;
      case CFGIN_VOLTAGE_2V5: cout << "CfgIn_getCfgInMode (" << (int)channel << "): CFGIN_VOLTAGE_2V5" << std::endl; break;
      case CFGIN_VOLTAGE_5V: cout << "CfgIn_getCfgInMode (" << (int)channel << "): CFGIN_VOLTAGE_5V" << std::endl; break;
      case CFGIN_RESISTANCE: cout << "CfgIn_getCfgInMode (" << (int)channel << "): CFGIN_RESISTANCE" << std::endl; break;
      case CFGIN_FREQ_FLOATING: cout << "CfgIn_getCfgInMode (" << (int)channel << "): CFGIN_FREQ_FLOATING" << std::endl; break;
      case CFGIN_FREQ_PULLUP: cout << "CfgIn_getCfgInMode (" << (int)channel << "): CFGIN_FREQ_PULLUP" << std::endl; break;
      case CFGIN_FREQ_PULLDOWN: cout << "CfgIn_getCfgInMode (" << (int)channel << "): CFGIN_FREQ_PULLDOWN" << std::endl; break;
      default: cout << "CfgIn_getCfgInMode (" << (int)channel << "): Unknown mode" << std::endl; break;
      }
    }
    //! [Calling CfgIn_getCfgInMode]

    channel++;
  }

}

int main(void)
{
  //! [Calling CfgIn_release]
  //! [Calling GetCfgIn]
  CFGINHANDLE pCfgIn = ::GetCfgIn();
  assert(pCfgIn);

  cfgin_example(pCfgIn);

  CfgIn_release(pCfgIn);
  //! [Calling GetCfgIn]
  //! [Calling CfgIn_release]
  
  return 0;
}
