/* g++ -DLINUX examples/config_example.cpp  -lcc-aux2 -pthread -ldl */
#include <Config.h>
#include <CCAuxErrors.h>
#include <assert.h>
#include <iostream>

using namespace std;
using namespace CrossControl;

void conf_example(CONFIGHANDLE pConfig)
{
  CrossControl::eErr err;
  CrossControl::TriggerConf trig;

  if (!pConfig)
    return;

  //! [Calling Config_getStartupTriggerConfig]
  err = Config_getStartupTriggerConfig(pConfig, &trig);
  if (err == ERR_SUCCESS) 
  {
    cout << "Start-up trigger is set to: ";
    switch(trig)
    {
    case Front_Button_Enabled: cout << "Front button only" << endl; break;
    case OnOff_Signal_Enabled:  cout << "On/Off signal only" << endl; break;
    case Both_Button_And_Signal_Enabled: cout << "Front button or On/off signal" << endl; break;

      // The below values are only available on the VC platform     
    case CAN_Button_Activity: cout << "Wake up on CAN and Buttons" << endl; break;
    case CAN_OnOff_Activity: cout << "Wake up on CAN and On/Off/Ignition signal" << endl; break;
    case CAN_Button_OnOff_Activity: cout << "Wake up on CAN, Buttons and On/Off/Ignition signal" << endl; break;
    case CI_Button_Activity: cout << "Wake up on CI and Button State Change" << endl; break;
    case CI_OnOff_Activity: cout << "Wake up on CI and OnOff Signal State Change" << endl; break;
    case CI_Button_OnOff_Activity: cout << "Wake up on CI, Button and OnOff Signal State Change" << endl; break;
    case CI_CAN_Button_Activity: cout << "Wake up on CI, CAN and Button State Change" << endl; break;
    case CI_CAN_OnOff_Activity: cout << "Wake up on CI, CAN and OnOff Signal State Change" << endl; break;
    case All_Events: cout << "Wake up on all events" << endl; break;
    default: cout << "Error - Undefined StartupTrigger" << endl; break;
    }
  }
  else
  {
    cout << "Error(" << err << ") in function getStartupTriggerConfig: " << GetErrorStringA(err) << endl;
  } 
  //! [Calling Config_getStartupTriggerConfig]

  //
  // Set the action to suspend mode when the front panel button is pressed (short time).
  //
  //! [Calling Config_setShortButtonPressAction]
  err = Config_setShortButtonPressAction(pConfig, ActionSuspend);
  if (err == ERR_SUCCESS) 
  {
    cout << "ShortButtonPressAction set to Suspend!" << endl;
  }
  else
  {
    cout << "Error(" << err << ") in function setShortButtonPressAction: " << GetErrorStringA(err) << endl;
  }
  //! [Calling Config_setShortButtonPressAction]
}

int main(void)
{
  //! [Calling Config_release]
  //! [Calling GetConfig]
  CONFIGHANDLE pConfig = ::GetConfig();
  assert(pConfig);

  conf_example(pConfig);

  Config_release(pConfig);
  //! [Calling GetConfig]
  //! [Calling Config_release]
  
  return 0;
}
