/* g++ -DLINUX examples/firmwareupgrade_example.cpp  -lcc-aux2 -pthread -ldl */
#include <assert.h>
#include <FirmwareUpgrade.h>
#include <iostream>
#ifdef LINUX
#include <cstring>
#else
#include <string>
#endif

using namespace std;
using namespace CrossControl;

void upgrade_ss(string path)
{
  const int max_retries = 3;
  CrossControl::eErr err;

  //! [Calling GetFirmwareUpgrade]
  FIRMWAREUPGHANDLE pFirmwareUpgrade = GetFirmwareUpgrade();
  assert(pFirmwareUpgrade != NULL);
  //! [Calling GetFirmwareUpgrade]

  //! [Calling FirmwareUpgrade_startSSUpgrade]
  //! [Calling FirmwareUpgrade_startSSVerification]
  cout << "Upgrading SS" << endl;

  for(int i=0;i<max_retries;i++)
  {
    // Reinitialize upgrade handle
    FirmwareUpgrade_release(pFirmwareUpgrade);
    pFirmwareUpgrade = GetFirmwareUpgrade();
    assert(pFirmwareUpgrade != NULL);

    err = FirmwareUpgrade_startSSUpgrade(pFirmwareUpgrade, path.c_str(), true);
    if (CrossControl::ERR_SUCCESS == err) 
    {
      cout << "Upgrade Ok" << endl;
      break;
    }
    else if (CrossControl::ERR_VERIFY_FAILED == err)
    {
      // Reinitialize upgrade handle
      FirmwareUpgrade_release(pFirmwareUpgrade);
      pFirmwareUpgrade = GetFirmwareUpgrade();
      assert(pFirmwareUpgrade != NULL);

      err = FirmwareUpgrade_startSSVerification(pFirmwareUpgrade, path.c_str(), true);

      if (CrossControl::ERR_SUCCESS == err) 
      {
        cout << "Upgrade Ok" << endl;
        break;
      }
    }
    else
    {
      cout << "Error " << err << " in function startSSUpgrade: " << GetErrorStringA(err) << std::endl;
    }    
  }
  //! [Calling FirmwareUpgrade_startSSUpgrade]
  //! [Calling FirmwareUpgrade_startSSVerification]

  FirmwareUpgrade_release(pFirmwareUpgrade);
}

void upgrade_front(string path)
{
  const int max_retries = 3;
  CrossControl::eErr err;

  FIRMWAREUPGHANDLE pFirmwareUpgrade = GetFirmwareUpgrade();
  assert(pFirmwareUpgrade != NULL);

  //! [Calling FirmwareUpgrade_startFrontUpgrade]
  //! [Calling FirmwareUpgrade_startFrontVerification]
  cout << "Upgrading front" << endl;

  for(int i=0;i<max_retries;i++)
  {
    // Reinitialize upgrade handle
    FirmwareUpgrade_release(pFirmwareUpgrade);
    pFirmwareUpgrade = GetFirmwareUpgrade();
    assert(pFirmwareUpgrade != NULL);

    err = FirmwareUpgrade_startFrontUpgrade(pFirmwareUpgrade, path.c_str(), true);
    if (CrossControl::ERR_SUCCESS == err)
    {
      cout << "Upgrade Ok" << endl;
      break;
    }
    else if (CrossControl::ERR_VERIFY_FAILED == err)
    {
      // Reinitialize upgrade handle
      FirmwareUpgrade_release(pFirmwareUpgrade);
      pFirmwareUpgrade = GetFirmwareUpgrade();
      assert(pFirmwareUpgrade != NULL);

      err = FirmwareUpgrade_startFrontVerification(pFirmwareUpgrade, path.c_str(), true);

      if (CrossControl::ERR_SUCCESS == err)
      {
        cout << "Upgrade Ok" << endl;
        break;
      }
    }
    else
    {
      cout << "Error " << err << " in function startFrontUpgrade: " << GetErrorStringA(err) << std::endl;
    }
  }
  //! [Calling FirmwareUpgrade_startFrontUpgrade]
  //! [Calling FirmwareUpgrade_startFrontVerification]

  FirmwareUpgrade_release(pFirmwareUpgrade);
}

void upgrade_fpga(string path)
{
  const int max_retries = 3;
  CrossControl::eErr err;

  FIRMWAREUPGHANDLE pFirmwareUpgrade = GetFirmwareUpgrade();
  assert(pFirmwareUpgrade != NULL);

  //! [Calling FirmwareUpgrade_startFpgaUpgrade]
  //! [Calling FirmwareUpgrade_startFpgaVerification]
  cout << "Upgrading FPGA" << endl;

  for(int i=0;i<max_retries;i++)
  {
    // Reinitialize upgrade handle
    FirmwareUpgrade_release(pFirmwareUpgrade);
    pFirmwareUpgrade = GetFirmwareUpgrade();
    assert(pFirmwareUpgrade != NULL);

    err = FirmwareUpgrade_startFpgaUpgrade(pFirmwareUpgrade, path.c_str(), true);
    if (CrossControl::ERR_SUCCESS == err)
    {
      cout << "Upgrade Ok" << endl;
      break;
    }
    else if (CrossControl::ERR_VERIFY_FAILED == err)
    {
      // Reinitialize upgrade handle
      FirmwareUpgrade_release(pFirmwareUpgrade);
      pFirmwareUpgrade = GetFirmwareUpgrade();
      assert(pFirmwareUpgrade != NULL);

      err = FirmwareUpgrade_startFpgaVerification(pFirmwareUpgrade, path.c_str(), true);

      if (CrossControl::ERR_SUCCESS == err) 
      {
        cout << "Upgrade Ok" << endl;
        break;
      }
    }
    else
    {
      cout << "Error " << err << " in function startFpgaUpgrade: " << GetErrorStringA(err) << std::endl;
    }
  }
  //! [Calling FirmwareUpgrade_startFpgaUpgrade]
  //! [Calling FirmwareUpgrade_startFpgaVerification]

  //! [Calling FirmwareUpgrade_release]
  FirmwareUpgrade_release(pFirmwareUpgrade);
  //! [Calling FirmwareUpgrade_release]
}

void usage(string progname)
{
  cout << "Usage: " << progname << 
    " <fpga|ss|front> <path>" << endl;
}

int main(int argc, char *argv[])
{
  if (argc != 3) 
  {
    usage(argv[0]);
    return -1;
  }

  if (!strcmp(argv[1], "fpga"))
  {
    upgrade_fpga(argv[2]);
  }
  else if (!strcmp(argv[1], "ss"))
  {
    upgrade_ss(argv[2]);
  }
  else if (!strcmp(argv[1], "front"))
  {
    upgrade_front(argv[2]);
  }
  else
  {
    usage(argv[0]);
  }

  return 0;
}
