/* g++ -DLINUX examples/Lightsensor_example.cpp  -lcc-aux2 -pthread -ldl */
#include <Lightsensor.h>
#include <CCAuxErrors.h>
#include <assert.h>
#include <iostream>

using namespace std;
using namespace CrossControl;

void ls_example(LIGHTSENSORHANDLE pLightSensor)
{
  CrossControl::eErr err;
  unsigned short value;

  //! [Calling Lightsensor_getIlluminance]
  err = Lightsensor_getIlluminance(pLightSensor, &value);
  if (err != ERR_SUCCESS)
  {
    cout << "Error(" << err << ") in function getIlluminance: " << GetErrorStringA(err) << endl;
  }
  //! [Calling Lightsensor_getIlluminance]

  cout << "Lightsensor value: " << (int)value << endl;

  cout << "Starting average calculation..." << endl;

  //! [Calling Lightsensor_startAverageCalc]
  // Start the average calculation background function
  // This cannot be used if the automatic backlihgt function is running.
  err = Lightsensor_startAverageCalc(pLightSensor, 5, 5, 50, SamplingModeAuto);
  if (err == ERR_AVERAGE_CALC_STARTED)
  {
    cout << "Error(" << err << ") in function startAverageCalc: " << GetErrorStringA(err) << endl;
    cout << endl << "Please turn off Automatic backlight! (CCsettings - Display tab)" << endl;
    return;
  }
  else if (err != ERR_SUCCESS)
  {
    cout << "Error(" << err << ") in function startAverageCalc: " << GetErrorStringA(err) << endl;
  }
  //! [Calling Lightsensor_startAverageCalc]

  cin.sync();
  cout << endl << "Press Enter to read the average value..." << endl;
  cin.get();

  //! [Calling Lightsensor_getAverageIlluminance]
  err = Lightsensor_getAverageIlluminance(pLightSensor, &value);
  if (err != ERR_SUCCESS)
  {
    cout << "Error(" << err << ") in function getAverageIlluminance: " << GetErrorStringA(err) << endl;
  }
  //! [Calling Lightsensor_getAverageIlluminance]

  cout << "Lightsensor average value: " << (int)value << endl;

  //! [Calling Lightsensor_stopAverageCalc]
  err = Lightsensor_stopAverageCalc(pLightSensor);
  if (err != ERR_SUCCESS)
  {
    cout << "Error(" << err << ") in function stopAverageCalc: " << GetErrorStringA(err) << endl;
  }
  //! [Calling Lightsensor_stopAverageCalc]
}

int main(void)
{
  //! [Calling Lightsensor_release]
  //! [Calling GetLightsensor]
  LIGHTSENSORHANDLE pLightSensor = ::GetLightsensor();
  assert(pLightSensor);

  ls_example(pLightSensor);

  Lightsensor_release(pLightSensor);
  //! [Calling GetLightsensor]
  //! [Calling Lightsensor_release]
  
  return 0;
}
