/* g++ -DLINUX examples/softkey_example.cpp  -lcc-aux2 -pthread -ldl */
#include <SoftKey.h>
#include <assert.h>
#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <bitset>
#include <string.h>

using namespace std;
using namespace CrossControl;

void softkey_example(SOFTKEYHANDLE pSoftKey)
{
  uint16_t value16;
  uint8_t value8;
  float64_t fvalue64;
  eErr err;
  uint8_t keys[4], intensities[4];
  
  //! [Calling SoftKey_getStatus]
  err = SoftKey_getStatus(pSoftKey, &value16);
  if (err == CrossControl::ERR_SUCCESS) {
    std::bitset<16> status(value16);
    cout << "SoftKey status " << status << std::endl;
  }
  //! [Calling SoftKey_getStatus] 

  //! [Calling SoftKey_setBacklightIntensity]
  err = SoftKey_setBacklightIntensity(pSoftKey, 1, 50);
  if (err == CrossControl::ERR_SUCCESS) 
    cout << "SoftKey 1 backlight set to 50" << std::endl;
  //! [Calling SoftKey_setBacklightIntensity]

  //! [Calling SoftKey_getBacklightIntensity]
  err = SoftKey_getBacklightIntensity(pSoftKey, 1, &value8);
  if (err == CrossControl::ERR_SUCCESS) 
    cout << "SoftKey 1 backlight intensity is " << std::dec << value8 << std::endl;
  //! [Calling SoftKey_getBacklightIntensity]

  //! [Calling SoftKey_setMultipleBacklightIntensities]
  keys[0] = 1;
  keys[1] = 4;
  keys[2] = 2;
  keys[3] = 3;
  intensities[0] = 25;
  intensities[1] = 50;
  intensities[2] = 75;
  intensities[3] = 100;
  err = SoftKey_setMultipleBacklightIntensities(pSoftKey, keys, intensities, sizeof(keys));
  if (err == CrossControl::ERR_SUCCESS)  {
    cout << "SoftKey set multiple keys intensities" << std::endl;
    cout << "SoftKey 1 intensity: " << +intensities[0] << std::endl;
    cout << "SoftKey 2 intensity: " << +intensities[3] << std::endl;
    cout << "SoftKey 3 intensity: " << +intensities[2] << std::endl;
    cout << "SoftKey 4 intensity: " << +intensities[1] << std::endl;
  }
  //! [Calling SoftKey_setMultipleBacklightIntensities]

  //! [Calling SoftKey_getMultipleBacklightIntensities]
  keys[0] = 1;
  keys[1] = 2;
  keys[2] = 3;
  keys[3] = 4;  
  memset(intensities, 0, sizeof(intensities));
  err = SoftKey_getMultipleBacklightIntensities(pSoftKey, keys, intensities, sizeof(keys));
  if (err == CrossControl::ERR_SUCCESS) {
    cout << "SoftKey get multiple keys intensities" << std::endl;
    cout << "SoftKey 1 intensity: " << +intensities[0] << std::endl;
    cout << "SoftKey 2 intensity: " << +intensities[1] << std::endl;
    cout << "SoftKey 3 intensity: " << +intensities[2] << std::endl;
    cout << "SoftKey 4 intensity: " << +intensities[3] << std::endl;
  }
  //! [Calling SoftKey_setMultipleBacklightIntensities]


  //! [Calling SoftKey_setBacklightSignal]
  err = SoftKey_setBacklightSignal(pSoftKey, 2, 50);
  if (err == CrossControl::ERR_SUCCESS) 
    cout << "SoftKey backlight signal set to 2Hz 50% duty cycle" << std::endl;
  //! [Calling SoftKey_setBacklightSignal]

  //! [Calling SoftKey_getBacklightSignal]
  err = SoftKey_getBacklightSignal(pSoftKey, &fvalue64, &value8);
  cout.precision(1);
  if (err == CrossControl::ERR_SUCCESS) 
    cout << "SoftKey backlight signal frequency is "<< std::fixed << fvalue64 << " and duty cycle " << std::dec << value8 << std::endl;
  //! [Calling SoftKey_getBacklightSignal]


  //! [Calling SoftKey_setBacklightOnTime]
  err = SoftKey_setBacklightOnTime(pSoftKey, 50);
  if (err == CrossControl::ERR_SUCCESS) 
    cout << "SoftKey backlight OnTime set to 50" << std::endl;  
  //! [Calling SoftKey_setBacklightOnTime]

  //! [Calling SoftKey_getBacklightOnTime]
  err = SoftKey_getBacklightOnTime(pSoftKey, &value8);
  if (err == CrossControl::ERR_SUCCESS) 
    cout << "SoftKey backlight OnTime is " << +value8 << std::endl;  
  //! [Calling SoftKey_getBacklightOnTime]

  //! [Calling SoftKey_setBacklightOffTime]
  err = SoftKey_setBacklightOffTime(pSoftKey, 50);
  if (err == CrossControl::ERR_SUCCESS) 
    cout << "SoftKey backlight OffTime set to 50" << std::endl;  
  //! [Calling SoftKey_setBacklightOffTime]

  //! [Calling SoftKey_getBacklightOffTime]
  err = SoftKey_getBacklightOffTime(pSoftKey, &value8);
  if (err == CrossControl::ERR_SUCCESS) 
    cout << "SoftKey backlight OffTime is " << +value8 << std::endl;  
  //! [Calling SoftKey_getBacklightOffTime]

  //! [Calling SoftKey_setBacklightIdleTime]
  err = SoftKey_setBacklightIdleTime(pSoftKey, 50);
  if (err == CrossControl::ERR_SUCCESS) 
    cout << "SoftKey backlight IdleTime set to 50" << std::endl;  
  //! [Calling SoftKey_setBacklightIdleTime]

  //! [Calling SoftKey_getBacklightIdleTime]
  err = SoftKey_getBacklightIdleTime(pSoftKey, &value8);
  if (err == CrossControl::ERR_SUCCESS) 
    cout << "SoftKey backlight IdleTime is " << +value8 << std::endl;  
  //! [Calling SoftKey_getBacklightIdleTime]
  
  //! [Calling SoftKey_setBacklightNrOfPulses]
  err = SoftKey_setBacklightNrOfPulses(pSoftKey, 50);
  if (err == CrossControl::ERR_SUCCESS) 
    cout << "SoftKey backlight NrOfPulses set to 50" << std::endl;  
  //! [Calling SoftKey_setBacklightNrOfPulses]

  //! [Calling SoftKey_getBacklightNrOfPulses]
  err = SoftKey_getBacklightNrOfPulses(pSoftKey, &value8);
  if (err == CrossControl::ERR_SUCCESS) 
    cout << "SoftKey backlight NrOfPulses is " << +value8 << std::endl;  
  //! [Calling SoftKey_getBacklightNrOfPulses]

  
}

int main(int argc, char* argv[])
{
  //! [Calling GetSoftKey] 
  //! [Calling SoftKey_release]
  SOFTKEYHANDLE pSoftKey = ::GetSoftKey();
  assert(pSoftKey);

  softkey_example(pSoftKey);

  SoftKey_release(pSoftKey);
  //! [Calling SoftKey_release]
  //! [Calling GetSoftKey]
  
  return 0;
}
