/* g++ -DLINUX examples/telematics_example.cpp  -lcc-aux2 -pthread -ldl */
#include <Telematics.h>
#include <CCAuxErrors.h>
#include <assert.h>
#include <iostream>

using namespace std;
using namespace CrossControl;

void telematics_example(TELEMATICSHANDLE pTelematics)
{
  CrossControl::eErr err;
  CCStatus status;

  //! [Calling Telematics_getTelematicsAvailable]
  err = Telematics_getTelematicsAvailable(pTelematics, &status);
  if (err == ERR_SUCCESS)
  {
    cout << "Telematics add-on board: " << ((status == Enabled)? "available" : "not available") << endl;
    if (status == Disabled)
      return;   
  } 
  else 
  {
    cout << "Error(" << err << ") in function getTelematicsAvailable: " << GetErrorStringA(err) << endl;
    return;
  }
  //! [Calling Telematics_getTelematicsAvailable]

  //! [Calling Telematics_getBTPowerStatus]
  err = Telematics_getBTPowerStatus(pTelematics, &status);
  if (err == ERR_SUCCESS)
  {
    cout << "Bluetooth power is " << ((status == Enabled)? "ON" : "OFF") << endl;
  }
  else if (err == ERR_TELEMATICS_BT_NOT_AVAILABLE)
  {
    cout << "getBLPowerStatus: Bluetooth is not available on this platform" << endl;
  }
  else 
  {
    cout << "Error(" << err << ") in function getBLPowerStatus: " << GetErrorStringA(err) << endl;
  }
  //! [Calling Telematics_getBTPowerStatus]

  //! [Calling Telematics_getBTStartUpPowerStatus]
  err = Telematics_getBTStartUpPowerStatus(pTelematics, &status);
  if (err == ERR_SUCCESS)
  {
    cout << "Bluetooth power is " << ((status == Enabled)? "Enabled" : "Disabled") << " at start-up" << endl;
  }
  else if (err == ERR_TELEMATICS_BT_NOT_AVAILABLE)
  {
    cout << "getBTStartUpPowerStatus: Bluetooth is not available on this platform" << endl;
  }
  else 
  {
    cout << "Error(" << err << ") in function getBTStartUpPowerStatus: " << GetErrorStringA(err) << endl;
  }
  //! [Calling Telematics_getBTStartUpPowerStatus]

  //! [Calling Telematics_getGPRSPowerStatus]
  err = Telematics_getGPRSPowerStatus(pTelematics, &status);
  if (err == ERR_SUCCESS)
  {
    cout << "GSM/GPRS power is " << ((status == Enabled)? "ON" : "OFF") << endl;
  }
  else if (err == ERR_TELEMATICS_GPRS_NOT_AVAILABLE)
  {
    cout << "getGPRSPowerStatus: GSM/GPRS is not available on this platform" << endl;
  }
  else 
  {
    cout << "Error(" << err << ") in function getGPRSPowerStatus: " << GetErrorStringA(err) << endl;
  }
  //! [Calling Telematics_getGPRSPowerStatus]

  //! [Calling Telematics_getGPRSStartUpPowerStatus]
  err = Telematics_getGPRSStartUpPowerStatus(pTelematics, &status);
  if (err == ERR_SUCCESS)
  {
    cout << "GSM/GPRS power is " << ((status == Enabled)? "Enabled" : "Disabled") << " at start-up" << endl;
  }
  else if (err == ERR_TELEMATICS_GPRS_NOT_AVAILABLE)
  {
    cout << "getGPRSStartUpPowerStatus: GSM/GPRS is not available on this platform" << endl;
  }
  else 
  {
    cout << "Error(" << err << ") in function getGPRSStartUpPowerStatus: " << GetErrorStringA(err) << endl;
  }
  //! [Calling Telematics_getGPRSStartUpPowerStatus]

  //! [Calling Telematics_getGPSPowerStatus]
  err = Telematics_getGPSPowerStatus(pTelematics, &status);
  if (err == ERR_SUCCESS)
  {
    cout << "GPS power is " << ((status == Enabled)? "ON" : "OFF") << endl;
  }
  else if (err == ERR_TELEMATICS_GPS_NOT_AVAILABLE)
  {
    cout << "getGPSPowerStatus: GPS is not available on this platform" << endl;
  }
  else 
  {
    cout << "Error(" << err << ") in function getGPSPowerStatus: " << GetErrorStringA(err) << endl;
  }
  //! [Calling Telematics_getGPSPowerStatus]

  //! [Calling Telematics_getGPSStartUpPowerStatus]
  err = Telematics_getGPSStartUpPowerStatus(pTelematics, &status);
  if (err == ERR_SUCCESS)
  {
    cout << "GPS power is " << ((status == Enabled)? "Enabled" : "Disabled") << " at start-up" << endl;
  }
  else if (err == ERR_TELEMATICS_GPS_NOT_AVAILABLE)
  {
    cout << "getGPSStartUpPowerStatus: GPS is not available on this platform" << endl;
  }
  else 
  {
    cout << "Error(" << err << ") in function getGPSStartUpPowerStatus: " << GetErrorStringA(err) << endl;
  }
  //! [Calling Telematics_getGPSStartUpPowerStatus]

  //! [Calling Telematics_getGPSAntennaStatus]
  err = Telematics_getGPSAntennaStatus(pTelematics, &status);
  if (err == ERR_SUCCESS)
  {
    cout << "GPS antenna status: " << ((status == Enabled)? "OK" : "ERROR: Open connection or short-circuit") << endl;
  }
  else if (err == ERR_TELEMATICS_GPS_NOT_AVAILABLE)
  {
    cout << "getGPSAntennaStatus: GPS is not available on this platform" << endl;
  }
  else 
  {
    cout << "Error(" << err << ") in function getGPSAntennaStatus: " << GetErrorStringA(err) << endl;
  }
  //! [Calling Telematics_getGPSAntennaStatus]

  //! [Calling Telematics_getWLANPowerStatus]
  err = Telematics_getWLANPowerStatus(pTelematics, &status);
  if (err == ERR_SUCCESS)
  {
    cout << "WLAN power is " << ((status == Enabled)? "ON" : "OFF") << endl;
  }
  else if (err == ERR_TELEMATICS_WLAN_NOT_AVAILABLE)
  {
    cout << "getWLANPowerStatus: WLAN is not available on this platform" << endl;
  }
  else 
  {
    cout << "Error(" << err << ") in function getWLANPowerStatus: " << GetErrorStringA(err) << endl;
  }
  //! [Calling Telematics_getWLANPowerStatus]

  //! [Calling Telematics_getWLANStartUpPowerStatus]
  err = Telematics_getWLANStartUpPowerStatus(pTelematics, &status);
  if (err == ERR_SUCCESS)
  {
    cout << "WLAN power is " << ((status == Enabled)? "Enabled" : "Disabled") << " at start-up" << endl;
  }
  else if (err == ERR_TELEMATICS_WLAN_NOT_AVAILABLE)
  {
    cout << "getWLANStartUpPowerStatus: WLAN is not available on this platform" << endl;
  }
  else 
  {
    cout << "Error(" << err << ") in function getWLANStartUpPowerStatus: " << GetErrorStringA(err) << endl;
  }
  //! [Calling Telematics_getWLANStartUpPowerStatus]
}

int main(void)
{
  //! [Calling Telematics_release]
  //! [Calling GetTelematics]
  TELEMATICSHANDLE pTelematics = ::GetTelematics();
  assert(pTelematics);

  telematics_example(pTelematics);

  Telematics_release(pTelematics);
  //! [Calling GetTelematics]
  //! [Calling Telematics_release]
  
  return 0;
}
